@@ -522,7 +522,7 @@ SEXP rnng_send_aio(SEXP con, SEXP data, SEXP mode, SEXP timeout, SEXP pipe, SEXP
     if (raw) {
       nano_encode(&buf, data);
     } else {
-      nano_serialize(&buf, data, NANO_PROT(con));
+      nano_serialize(&buf, data, NANO_PROT(con), 0);
     }
     nng_msg *msg = NULL;
 
