@@ -1,19 +1,3 @@
-// Copyright (C) 2022-2025 Hibiki AI Limited <info@hibiki-ai.com>
-//
-// This file is part of nanonext.
-//
-// nanonext is free software: you can redistribute it and/or modify it under the
-// terms of the GNU General Public License as published by the Free Software
-// Foundation, either version 3 of the License, or (at your option) any later
-// version.
-//
-// nanonext is distributed in the hope that it will be useful, but WITHOUT ANY
-// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-// A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License along with
-// nanonext. If not, see <https://www.gnu.org/licenses/>.
-
 // nanonext - C level - Socket and Stream Constructors -------------------------
 
 #define NANONEXT_PROTOCOLS
@@ -36,7 +20,7 @@ static void stream_finalizer(SEXP xptr) {
   }
   if (xp->tls != NULL)
     nng_tls_config_free(xp->tls);
-  R_Free(xp);
+  free(xp);
 
 }
 
@@ -50,41 +34,38 @@ SEXP rnng_protocol_open(SEXP protocol, SEXP dial, SEXP listen, SEXP tls, SEXP au
 
   const char *pname;
   int xc;
-  nng_socket *sock;
   SEXP socket;
 
+  nng_socket *sock = malloc(sizeof(nng_socket));
+  NANO_ENSURE_ALLOC(sock);
+
   switch (slen) {
   case 1:
   case 2:
   case 3:
     if (!memcmp(pro, "bus", slen)) {
       pname = "bus";
-      sock = R_Calloc(1, nng_socket);
       xc = rw ? nng_bus0_open_raw(sock) : nng_bus0_open(sock);
       break;
     }
     if (slen > 2) {
       if (!memcmp(pro, "pub", slen)) {
         pname = "pub";
-        sock = R_Calloc(1, nng_socket);
         xc = rw ? nng_pub0_open_raw(sock) : nng_pub0_open(sock);
         break;
       }
       if (!memcmp(pro, "sub", slen)) {
         pname = "sub";
-        sock = R_Calloc(1, nng_socket);
         xc = rw ? nng_sub0_open_raw(sock) : nng_sub0_open(sock);
         break;
       }
       if (!memcmp(pro, "req", slen)) {
         pname = "req";
-        sock = R_Calloc(1, nng_socket);
         xc = rw ? nng_req0_open_raw(sock) : nng_req0_open(sock);
         break;
       }
       if (!memcmp(pro, "rep", slen)) {
         pname = "rep";
-        sock = R_Calloc(1, nng_socket);
         xc = rw ? nng_rep0_open_raw(sock) : nng_rep0_open(sock);
         break;
       }
@@ -93,26 +74,22 @@ SEXP rnng_protocol_open(SEXP protocol, SEXP dial, SEXP listen, SEXP tls, SEXP au
     if (slen > 1) {
       if (!memcmp(pro, "pair", slen)) {
         pname = "pair";
-        sock = R_Calloc(1, nng_socket);
         xc = rw ? nng_pair0_open_raw(sock) : nng_pair0_open(sock);
         break;
       }
       if (!memcmp(pro, "poly", slen)) {
         pname = "poly";
-        sock = R_Calloc(1, nng_socket);
         xc = rw ? nng_pair1_open_raw(sock) : nng_pair1_open_poly(sock);
         break;
       }
       if (slen > 2) {
         if (!memcmp(pro, "push", slen)) {
           pname = "push";
-          sock = R_Calloc(1, nng_socket);
           xc = rw ? nng_push0_open_raw(sock) : nng_push0_open(sock);
           break;
         }
         if (!memcmp(pro, "pull", slen)) {
           pname = "pull";
-          sock = R_Calloc(1, nng_socket);
           xc = rw ? nng_pull0_open_raw(sock) : nng_pull0_open(sock);
           break;
         }
@@ -124,7 +101,6 @@ SEXP rnng_protocol_open(SEXP protocol, SEXP dial, SEXP listen, SEXP tls, SEXP au
   case 8:
     if (slen > 2 && !memcmp(pro, "surveyor", slen)) {
       pname = "surveyor";
-      sock = R_Calloc(1, nng_socket);
       xc = rw ? nng_surveyor0_open_raw(sock) : nng_surveyor0_open(sock);
       break;
     }
@@ -132,18 +108,16 @@ SEXP rnng_protocol_open(SEXP protocol, SEXP dial, SEXP listen, SEXP tls, SEXP au
   case 10:
     if (slen > 2 && !memcmp(pro, "respondent", slen)) {
       pname = "respondent";
-      sock = R_Calloc(1, nng_socket);
       xc = rw ? nng_respondent0_open_raw(sock) : nng_respondent0_open(sock);
       break;
     }
   default:
-    NANO_ERROR("'protocol' should be one of bus, pair, poly, push, pull, pub, sub, req, rep, surveyor, respondent");
+    free(sock);
+    Rf_error("`protocol` should be one of: bus, pair, poly, push, pull, pub, sub, req, rep, surveyor, respondent");
   }
 
-  if (xc) {
-    R_Free(sock);
-    ERROR_OUT(xc);
-  }
+  if (xc)
+    goto fail;
 
   PROTECT(socket = R_MakeExternalPtr(sock, nano_SocketSymbol, R_NilValue));
   R_RegisterCFinalizerEx(socket, socket_finalizer, TRUE);
@@ -153,21 +127,34 @@ SEXP rnng_protocol_open(SEXP protocol, SEXP dial, SEXP listen, SEXP tls, SEXP au
   Rf_setAttrib(socket, nano_ProtocolSymbol, Rf_mkString(pname));
   Rf_setAttrib(socket, nano_StateSymbol, Rf_mkString("opened"));
 
-  if (dial != R_NilValue)
-    rnng_dial(socket, dial, tls, autostart, Rf_ScalarLogical(1));
+  if (dial != R_NilValue) {
+    SEXP intd;
+    PROTECT(intd = Rf_ScalarInteger(2));
+    rnng_dial(socket, dial, tls, autostart, intd);
+    UNPROTECT(1);
+  }
 
-  if (listen != R_NilValue)
-    rnng_listen(socket, listen, tls, autostart, Rf_ScalarLogical(1));
+  if (listen != R_NilValue) {
+    SEXP intl;
+    PROTECT(intl = Rf_ScalarInteger(2));
+    rnng_listen(socket, listen, tls, autostart, intl);
+    UNPROTECT(1);
+  }
 
   UNPROTECT(1);
   return socket;
 
+  fail:
+  free(sock);
+  failmem:
+  ERROR_OUT(xc);
+
 }
 
 SEXP rnng_close(SEXP socket) {
 
   if (NANO_PTR_CHECK(socket, nano_SocketSymbol))
-    Rf_error("'socket' is not a valid Socket");
+    Rf_error("`socket` is not a valid Socket");
 
   nng_socket *sock = (nng_socket *) NANO_PTR(socket);
   const int xc = nng_close(*sock);
@@ -212,40 +199,38 @@ static SEXP nano_stream_dial(SEXP url, SEXP textframes, SEXP tls) {
 
   const char *add = CHAR(STRING_ELT(url, 0));
   if (tls != R_NilValue && NANO_PTR_CHECK(tls, nano_TlsSymbol))
-    Rf_error("'tls' is not a valid TLS Configuration");
-  nano_stream *nst = R_Calloc(1, nano_stream);
-  nst->mode = NANO_STREAM_DIALER;
-  nst->textframes = NANO_INTEGER(textframes) != 0;
-  nst->tls = NULL;
-  nng_url *up;
-  nng_aio *aiop;
+    Rf_error("`tls` is not a valid TLS Configuration");
+
+  nng_url *up = NULL;
+  nng_aio *aiop = NULL;
   int xc;
   SEXP sd;
 
-  if ((xc = nng_url_parse(&up, add)))
-    goto exitlevel1;
+  nano_stream *nst = calloc(1, sizeof(nano_stream));
+  NANO_ENSURE_ALLOC(nst);
+  nst->mode = NANO_STREAM_DIALER;
+  nst->textframes = NANO_INTEGER(textframes) != 0;
 
-  xc = nng_stream_dialer_alloc_url(&nst->endpoint.dial, up);
-  if (xc)
-    goto exitlevel2;
+  if ((xc = nng_url_parse(&up, add)) ||
+      (xc = nng_stream_dialer_alloc_url(&nst->endpoint.dial, up)) ||
+      (xc = nng_aio_alloc(&aiop, NULL, NULL)))
+    goto fail;
 
   if (!strcmp(up->u_scheme, "ws") || !strcmp(up->u_scheme, "wss")) {
     if (nst->textframes &&
         ((xc = nng_stream_dialer_set_bool(nst->endpoint.dial, "ws:recv-text", 1)) ||
         (xc = nng_stream_dialer_set_bool(nst->endpoint.dial, "ws:send-text", 1))))
-      goto exitlevel3;
+      goto fail;
   }
 
   if (!strcmp(up->u_scheme, "wss")) {
 
     if (tls == R_NilValue) {
-      if ((xc = nng_tls_config_alloc(&nst->tls, NNG_TLS_MODE_CLIENT)))
-        goto exitlevel3;
-
-      if ((xc = nng_tls_config_server_name(nst->tls, up->u_hostname)) ||
+      if ((xc = nng_tls_config_alloc(&nst->tls, NNG_TLS_MODE_CLIENT)) ||
+          (xc = nng_tls_config_server_name(nst->tls, up->u_hostname)) ||
           (xc = nng_tls_config_auth_mode(nst->tls, NNG_TLS_AUTH_MODE_NONE)) ||
           (xc = nng_stream_dialer_set_ptr(nst->endpoint.dial, NNG_OPT_TLS_CONFIG, nst->tls)))
-        goto exitlevel4;
+        goto fail;
     } else {
 
       nst->tls = (nng_tls_config *) NANO_PTR(tls);
@@ -253,18 +238,15 @@ static SEXP nano_stream_dial(SEXP url, SEXP textframes, SEXP tls) {
 
       if ((xc = nng_tls_config_server_name(nst->tls, up->u_hostname)) ||
           (xc = nng_stream_dialer_set_ptr(nst->endpoint.dial, NNG_OPT_TLS_CONFIG, nst->tls)))
-        goto exitlevel4;
+        goto fail;
     }
 
   }
 
-  if ((xc = nng_aio_alloc(&aiop, NULL, NULL)))
-    goto exitlevel4;
-
   nng_stream_dialer_dial(nst->endpoint.dial, aiop);
   nng_aio_wait(aiop);
   if ((xc = nng_aio_result(aiop)))
-    goto exitlevel5;
+    goto fail;
 
   nst->stream = nng_aio_get_output(aiop, 0);
 
@@ -282,17 +264,14 @@ static SEXP nano_stream_dial(SEXP url, SEXP textframes, SEXP tls) {
   UNPROTECT(1);
   return sd;
 
-  exitlevel5:
-  nng_aio_free(aiop);
-  exitlevel4:
+  fail:
   if (nst->tls != NULL)
     nng_tls_config_free(nst->tls);
-  exitlevel3:
+  nng_aio_free(aiop);
   nng_stream_dialer_free(nst->endpoint.dial);
-  exitlevel2:
   nng_url_free(up);
-  exitlevel1:
-  R_Free(nst);
+  free(nst);
+  failmem:
   ERROR_OUT(xc);
 
 }
@@ -301,39 +280,37 @@ static SEXP nano_stream_listen(SEXP url, SEXP textframes, SEXP tls) {
 
   const char *add = CHAR(STRING_ELT(url, 0));
   if (tls != R_NilValue && NANO_PTR_CHECK(tls, nano_TlsSymbol))
-    Rf_error("'tls' is not a valid TLS Configuration");
-  nano_stream *nst = R_Calloc(1, nano_stream);
-  nst->mode = NANO_STREAM_LISTENER;
-  nst->textframes = NANO_INTEGER(textframes) != 0;
-  nst->tls = NULL;
-  nng_url *up;
-  nng_aio *aiop;
+    Rf_error("`tls` is not a valid TLS Configuration");
+
+  nng_url *up = NULL;
+  nng_aio *aiop = NULL;
   int xc;
   SEXP sl;
 
-  if ((xc = nng_url_parse(&up, add)))
-    goto exitlevel1;
+  nano_stream *nst = calloc(1, sizeof(nano_stream));
+  NANO_ENSURE_ALLOC(nst);
+  nst->mode = NANO_STREAM_LISTENER;
+  nst->textframes = NANO_INTEGER(textframes) != 0;
 
-  xc = nng_stream_listener_alloc_url(&nst->endpoint.list, up);
-  if (xc)
-    goto exitlevel2;
+  if ((xc = nng_url_parse(&up, add)) ||
+      (xc = nng_stream_listener_alloc_url(&nst->endpoint.list, up)) ||
+      (xc = nng_aio_alloc(&aiop, NULL, NULL)))
+    goto fail;
 
   if (!strcmp(up->u_scheme, "ws") || !strcmp(up->u_scheme, "wss")) {
     if (nst->textframes &&
         ((xc = nng_stream_listener_set_bool(nst->endpoint.list, "ws:recv-text", 1)) ||
         (xc = nng_stream_listener_set_bool(nst->endpoint.list, "ws:send-text", 1))))
-      goto exitlevel3;
+      goto fail;
   }
 
   if (!strcmp(up->u_scheme, "wss")) {
 
     if (tls == R_NilValue) {
-      if ((xc = nng_tls_config_alloc(&nst->tls, NNG_TLS_MODE_SERVER)))
-        goto exitlevel3;
-
-      if ((xc = nng_tls_config_auth_mode(nst->tls, NNG_TLS_AUTH_MODE_NONE)) ||
+      if ((xc = nng_tls_config_alloc(&nst->tls, NNG_TLS_MODE_SERVER)) ||
+          (xc = nng_tls_config_auth_mode(nst->tls, NNG_TLS_AUTH_MODE_NONE)) ||
           (xc = nng_stream_listener_set_ptr(nst->endpoint.list, NNG_OPT_TLS_CONFIG, nst->tls)))
-        goto exitlevel4;
+        goto fail;
     } else {
 
       nst->tls = (nng_tls_config *) NANO_PTR(tls);
@@ -341,21 +318,18 @@ static SEXP nano_stream_listen(SEXP url, SEXP textframes, SEXP tls) {
 
       if ((xc = nng_tls_config_server_name(nst->tls, up->u_hostname)) ||
           (xc = nng_stream_listener_set_ptr(nst->endpoint.list, NNG_OPT_TLS_CONFIG, nst->tls)))
-        goto exitlevel4;
+        goto fail;
     }
 
   }
 
   if ((xc = nng_stream_listener_listen(nst->endpoint.list)))
-    goto exitlevel4;
-
-  if ((xc = nng_aio_alloc(&aiop, NULL, NULL)))
-    goto exitlevel4;
+    goto fail;
 
   nng_stream_listener_accept(nst->endpoint.list, aiop);
   nng_aio_wait(aiop);
   if ((xc = nng_aio_result(aiop)))
-    goto exitlevel5;
+    goto fail;
 
   nst->stream = nng_aio_get_output(aiop, 0);
 
@@ -373,35 +347,33 @@ static SEXP nano_stream_listen(SEXP url, SEXP textframes, SEXP tls) {
   UNPROTECT(1);
   return sl;
 
-  exitlevel5:
-  nng_aio_free(aiop);
-  exitlevel4:
+  fail:
   if (nst->tls != NULL)
     nng_tls_config_free(nst->tls);
-  exitlevel3:
+  nng_aio_free(aiop);
   nng_stream_listener_free(nst->endpoint.list);
-  exitlevel2:
   nng_url_free(up);
-  exitlevel1:
-  R_Free(nst);
+  free(nst);
+  failmem:
   ERROR_OUT(xc);
 
 }
 
 SEXP rnng_stream_open(SEXP dial, SEXP listen, SEXP textframes, SEXP tls) {
+
   if (dial != R_NilValue) {
     return nano_stream_dial(dial, textframes, tls);
   } else if (listen != R_NilValue) {
     return nano_stream_listen(listen, textframes, tls);
-  } else {
-    NANO_ERROR("specify a URL for either 'dial' or 'listen'");
   }
+  Rf_error("specify a URL for either `dial` or `listen`");
+
 }
 
 SEXP rnng_stream_close(SEXP stream) {
 
   if (NANO_PTR_CHECK(stream, nano_StreamSymbol))
-    Rf_error("'stream' is not a valid Stream");
+    Rf_error("`stream` is not a valid Stream");
 
   stream_finalizer(stream);
   R_ClearExternalPtr(stream);
