@@ -213,7 +213,6 @@ test_error(req$opt("false", list()), "type")
 
 test_class("nanoContext", ctx <- context(rep))
 test_print(ctx)
-test_equal(.header(12345L), 12345L)
 test_true(.mark())
 test_class("sendAio", csaio <- req$send_aio(data.frame(), mode = 1L, timeout = 500))
 test_zero(call_aio_(csaio)$result)
@@ -221,10 +220,8 @@ test_class("recvAio", craio <- recv_aio(ctx, mode = 8L, timeout = 500))
 test_type("raw", res <- collect_aio(craio))
 test_true(.read_marker(res))
 test_true(!.read_marker("not"))
-test_equal(.read_header(res), 12345L)
 test_equal(.read_header("not"), 0L)
 test_type("list", unserialize(res[9:length(res)]))
-test_equal(.header(0L), 0L)
 test_true(!.mark(FALSE))
 test_zero(req$send("context test", mode ="raw", block = 500))
 test_equal(recv(ctx, mode = "string", block = 500), "context test")
@@ -234,7 +231,6 @@ test_type("logical", .unresolved(msg))
 test_type("logical", unresolved(msg))
 test_class("data.frame", call_aio(msg)$data)
 test_true(!unresolved(msg))
-test_equal(.header(2025250L), 2025250L)
 test_zero(req$send(c(TRUE, FALSE, TRUE), mode = 2L, block = 500))
 test_class("recvAio", msg <- recv_aio(ctx, mode = 6L, timeout = 500))
 test_type("logical", msg[])
@@ -260,7 +256,6 @@ test_class("recvAio", rek <- request(req$context, c(1+3i, 4+2i), send_mode = "se
 test_zero(reply(ctx, execute = identity, recv_mode = 1L, send_mode = 1L, timeout = 500))
 test_type("complex", call_aio(rek)[["data"]])
 test_type("integer", rek[["aio"]])
-test_equal(.header(0L), 0L)
 
 test_type("list", cfg <- serial_config(class = c("invalid", "custom"), sfunc = list(identity, function(x) raw(1L)), ufunc = list(identity, as.integer)))
 opt(req$socket, "serial") <- cfg
@@ -286,19 +281,19 @@ test_error(serial_config("custom", identity, "func2"), "must be a function or li
 test_error(opt(rep, "wrong") <- cfg, "not supported")
 test_error(opt(rep, "serial") <- pairlist(a = 1L), "not supported")
 
-test_class("recvAio", cs <- request(req$context, "test", send_mode = "serial", cv = cv, timeout = 500))
+test_class("recvAio", cs <- request(req$context, "test", send_mode = "serial", cv = cv, timeout = 500, id = 12345L))
 test_notnull(cs$data)
 test_type("externalptr", ctxn <- .context(rep))
-test_class("recvAio", cr <- recv_aio(ctxn, cv = cv, timeout = 500))
-test_equal(call_aio(cr)$data, "test")
+test_class("recvAio", cr <- recv_aio(ctxn, mode = 8L, cv = cv, timeout = 500))
+test_equal(.read_header(call_aio(cr)$data), 12345L)
 test_type("integer", cr$aio)
 test_type("integer", send(ctxn, TRUE, mode = 0L, block = FALSE))
 test_type("externalptr", ctxn <- .context(rep))
-test_class("recvAio", cs <- request(.context(req$socket), data = TRUE, cv = NA))
+test_class("recvAio", cs <- request(.context(req$socket), data = TRUE, cv = NA, id = TRUE))
 test_notnull(cs$data)
 test_true(recv(ctxn, block = 500))
 test_zero(send(ctxn, TRUE, mode = 1L, block = 500))
-test_class("recvAio", cs <- request(.context(req$socket), data = TRUE, timeout = 5, id = 12L))
+test_class("recvAio", cs <- request(.context(req$socket), data = TRUE, timeout = 5, id = TRUE))
 test_zero(reap(ctxn))
 test_equal(reap(ctxn), 7L)
 test_zero(pipe_notify(rep, cv, add = TRUE, flag = TRUE))
@@ -435,7 +430,7 @@ test_error(socket(listen = "test"), "argument")
 test_type("list", ncurl("http://www.cam.ac.uk/"))
 test_type("list", ncurl("http://www.cam.ac.uk/", follow = FALSE, response = "date"))
 test_type("list", ncurl("http://www.cam.ac.uk/", follow = TRUE))
-test_type("list", ncurl("http://postman-echo.com/post", convert = FALSE, method = "POST", headers = c(`Content-Type` = "text/plain"), data = "test", response = c("Date", "Server"), timeout = 3000))
+test_type("list", ncurl("https://postman-echo.com/post", convert = FALSE, method = "POST", headers = c(`Content-Type` = "application/octet-stream"), data = as.raw(1L), response = c("Date", "Server"), timeout = 3000))
 test_class("errorValue", ncurl("http")$data)
 test_class("recvAio", haio <- ncurl_aio("http://example.com/"))
 test_true(is_aio(haio))
@@ -443,7 +438,7 @@ test_type("integer", call_aio(haio)$status)
 test_class("ncurlAio", haio <- ncurl_aio("https://example.com/", convert = FALSE, response = "server"))
 test_notnull(haio$status)
 if (call_aio(haio)$status == 200L) test_notnull(haio$headers)
-test_class("ncurlAio", put1 <- ncurl_aio("http://postman-echo.com/put", method = "PUT", headers = c(Authorization = "Bearer token"), data = "test", response = c("Date", "server"), timeout = 3000L))
+test_class("ncurlAio", put1 <- ncurl_aio("https://postman-echo.com/put", method = "PUT", headers = c(`Content-Type` = "text/plain", Authorization = "Bearer token"), data = "test", response = c("Date", "server"), timeout = 3000L))
 test_print(put1)
 test_type("integer", call_aio_(put1)$status)
 if (put1$status == 200L) test_notnull(put1$headers)
